//
//  ViewController.swift
//  DCMotor
//
//  Created by Phidgets on 2018-01-26.
//  Copyright © 2018 Phidgets. All rights reserved.
//

import UIKit
import Phidget22Swift

class ViewController: UIViewController {

    let ch = DCMotor()
    var phidgetInfoBox:PhidgetInfoBoxViewController! = nil
    
    @IBOutlet var velocityLabel: UILabel!
    @IBOutlet var backEMFTitle: UILabel!
    @IBOutlet var backEMFLabel: UILabel!
    @IBOutlet var targetVelocityTitle: UILabel!
    @IBOutlet var targetVelocityLabel: UILabel!
    @IBOutlet var targetVelocitySlider: UISlider!
    @IBOutlet var accelerationTitle: UILabel!
    @IBOutlet var accelerationLabel: UILabel!
    @IBOutlet var accelerationSlider: UISlider!
    @IBOutlet var brakingStrengthTitle: UILabel!
    @IBOutlet var brakingStrengthLabel: UILabel!
    @IBOutlet var brakingStrengthSlider: UISlider!
    @IBOutlet var currentLimitTitle: UILabel!
    @IBOutlet var currentLimitLabel: UILabel!
    @IBOutlet var currentLimitSlider: UISlider!
    @IBOutlet var currentRegulatorTitle: UILabel!
    @IBOutlet var currentRegulatorSlider: UISlider!
    @IBOutlet var currentRegulatorLabel: UILabel!
    @IBOutlet var enableBackEMFButton: UIButton!
    @IBOutlet var fanModeSegmentedControl: UISegmentedControl!
    @IBOutlet var stackView: UIStackView!
    
    
    override func viewDidLoad() {
        super.viewDidLoad()
        do {
            try Net.enableServerDiscovery(serverType: .deviceRemote)
            let _ = ch.attach.addHandler(attach_handler)
            let _ = ch.detach.addHandler(detach_handler)
            let _ = ch.error.addHandler(error_handler)
            let _ = ch.backEMFChange.addHandler(backemfchange_handler)
            let _ = ch.velocityUpdate.addHandler(velocityupdate_handler)
            let _ = ch.brakingStrengthChange.addHandler(brakingstrengthchange_handler)
            try ch.open()
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    
    //MARK: Event Handlers
    func attach_handler(sender: Phidget){
        do{
            try phidgetInfoBox.fillPhidgetInfo(phid: sender)
            let attachedDevice = sender as! DCMotor
            let deviceID = try attachedDevice.getDeviceID()
            
            let minTargetVelocity = try Float(attachedDevice.getMaxVelocity() * -1)
            let maxTargetVelocity = try Float(attachedDevice.getMaxVelocity())
            let minAcceleration = try Float(attachedDevice.getMinAcceleration())
            let maxAcceleration = try Float(attachedDevice.getMaxAcceleration())
            let acceleration = try Float(attachedDevice.getAcceleration())
            
            switch(deviceID){
            case DeviceID.PN_1065:
                let backEMFSensingState = try attachedDevice.getBackEMFSensingState()
                DispatchQueue.main.async {
                    self.enableBackEMFButton.setTitle(backEMFSensingState ? "Disable BackEMF Sensing" : "Enable BackEMFSensing", for: UIControlState.normal)
                }
                updateGUI(backEMFIsHidden: false, brakingStrengthIsHidden: false, currentLimitIsHidden: true, currentRegulatorIsHidden: true, fanStateIsHidden: true)
                break
            case DeviceID.PN_DCC1000:
                let minBrakingStrength = try Float(attachedDevice.getMinBrakingStrength())
                let maxBrakingStrength = try Float(attachedDevice.getMaxBrakingStrength())
                let brakingStrength = try Float(attachedDevice.getTargetBrakingStrength())
                let minCurrentLimit = try Float(attachedDevice.getMinCurrentLimit())
                let maxCurrentLimit = try Float(attachedDevice.getMaxCurrentLimit())
                let currentLimit = try Float(attachedDevice.getCurrentLimit())
                let minCurrentRegulatorGain = try Float(attachedDevice.getMinCurrentRegulatorGain())
                let maxCurrentRegulatorGain = try Float(attachedDevice.getMaxCurrentRegulatorGain())
                let currentRegulatorGain = try Float(attachedDevice.getCurrentRegulatorGain())
                let fanMode = try attachedDevice.getFanMode()
                
                updateGUI(backEMFIsHidden: true, brakingStrengthIsHidden: false, currentLimitIsHidden: false, currentRegulatorIsHidden: false, fanStateIsHidden: false)
                
                DispatchQueue.main.async {
                    self.brakingStrengthSlider.minimumValue = minBrakingStrength
                    self.brakingStrengthSlider.maximumValue = maxBrakingStrength
                    self.brakingStrengthSlider.value = brakingStrength
                    self.brakingStrengthLabel.text = String(format:"%.2f", brakingStrength)
                    
                    self.currentLimitSlider.minimumValue = minCurrentLimit
                    self.currentLimitSlider.maximumValue = maxCurrentLimit
                    self.currentLimitSlider.value = currentLimit
                    self.currentLimitLabel.text = String(format:"%.2f A", currentLimit)
                    
                    self.currentRegulatorSlider.minimumValue = minCurrentRegulatorGain
                    self.currentRegulatorSlider.maximumValue = maxCurrentRegulatorGain
                    self.currentRegulatorSlider.value = currentRegulatorGain
                    self.currentRegulatorLabel.text = String(format:"%.2f", currentRegulatorGain)
                    
                    self.fanModeSegmentedControl.selectedSegmentIndex = (fanMode == FanMode.auto) ? 0 : (fanMode == FanMode.on) ? 1 : 2;
                }
                break
            case DeviceID.PN_1060:
                fallthrough
            case DeviceID.PN_1064:
                fallthrough
            default:
                updateGUI(backEMFIsHidden: true, brakingStrengthIsHidden: true, currentLimitIsHidden: true, currentRegulatorIsHidden: true, fanStateIsHidden: true)
                break
                
                
            }
            
            DispatchQueue.main.async {
                self.stackView.isHidden = false
                self.targetVelocitySlider.minimumValue = minTargetVelocity
                self.targetVelocitySlider.maximumValue = maxTargetVelocity
                self.targetVelocitySlider.value = 0
                self.targetVelocityLabel.text = "0.0"
                
                self.accelerationSlider.minimumValue = minAcceleration
                self.accelerationSlider.maximumValue = maxAcceleration
                self.accelerationSlider.value = acceleration
                self.accelerationLabel.text = String(format:"%.2f", acceleration)
                
                
            }
            
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
        
    }
    
    func detach_handler(sender: Phidget){
        do{
            try phidgetInfoBox.fillPhidgetInfo(phid: nil)
            DispatchQueue.main.async {
                self.stackView.isHidden = true
            }
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    
    func error_handler(sender: Phidget, data: (code: ErrorEventCode, description: String)){
        self.outputErrorMessage(errorTitle: "Error Code: \(data.code)", errorMessage: data.description)
    }
    var timeStamp:TimeInterval = 0
    var lastTimeStamp:TimeInterval = 0
    func outputErrorMessage(errorTitle:String, errorMessage:String){
        timeStamp = NSDate.timeIntervalSinceReferenceDate
        //Add a delay for error events so the user does not get bombarded by error events
        if(timeStamp - lastTimeStamp > 5){
            let alert = UIAlertController(title: errorTitle, message: errorMessage, preferredStyle: .alert)
            alert.addAction(UIAlertAction(title: "OK", style: .default, handler: {action -> Void in
            }))
            self.present(alert, animated: true, completion: nil)
            lastTimeStamp = timeStamp
        }
    }
    
    func velocityupdate_handler(sender: DCMotor, velocity: Double){
        DispatchQueue.main.async {
            self.velocityLabel.text = String(format:"%0.2f", velocity)
        }
    }
    
    func brakingstrengthchange_handler(sender: DCMotor, brakingStrength: Double){
        DispatchQueue.main.async {
            self.brakingStrengthLabel.text = String(format:"%0.2f", brakingStrength)
        }
    }
    
    func backemfchange_handler(sender: DCMotor, backEMF: Double){
        DispatchQueue.main.async {
            self.backEMFLabel.text = String(format:"%0.2f", backEMF)
        }
    }
    
    //GUI controls
    @IBAction func setTargetVelocity(_ sender: Any) {
        let slider = sender as! UISlider
        do{
            try ch.setTargetVelocity(Double(slider.value))
            DispatchQueue.main.async {
                self.targetVelocityLabel.text = String(format:"%.2f", Double(slider.value))
            }
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    @IBAction func setAcceleration(_ sender: Any) {
        let slider = sender as! UISlider
        do{
            try ch.setAcceleration(Double(slider.value))
            DispatchQueue.main.async {
                self.accelerationLabel.text = String(format:"%.2f", Double(slider.value))
            }
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    @IBAction func setBrakingStrength(_ sender: Any) {
        let slider = sender as! UISlider
        do{
            try ch.setTargetBrakingStrength(Double(slider.value))
            DispatchQueue.main.async {
                self.brakingStrengthLabel.text = String(format:"%.2f", Double(slider.value))
            }
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    @IBAction func setCurrentLimit(_ sender: Any) {
        let slider = sender as! UISlider
        do{
            try ch.setCurrentLimit(Double(slider.value))
            DispatchQueue.main.async {
                self.currentLimitLabel.text = String(format:"%.2f A", Double(slider.value))
            }
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    @IBAction func setCurrentRegulatorGain(_ sender: Any) {
        let slider = sender as! UISlider
        do{
            try ch.setCurrentRegulatorGain(Double(slider.value))
            DispatchQueue.main.async {
                self.currentRegulatorLabel.text = String(format:"%.2f", Double(slider.value))
            }
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    @IBAction func setFanMode(_ sender: Any) {
        let segment = sender as! UISegmentedControl
        do{
            switch(segment.selectedSegmentIndex){
            case 0:
                try ch.setFanMode(FanMode.auto)
                break
            case 1:
                try ch.setFanMode(FanMode.on)
                break
            case 2:
                try ch.setFanMode(FanMode.off)
                break
            default:
                break
            }
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    @IBAction func enableBackEMFSensing(_ sender: Any) {
        let button = sender as! UIButton
        do{
            let state = try ch.getBackEMFSensingState()
            try ch.setBackEMFSensingState(!state)
            if(state){
                button.setTitle("Enable BackEMF Sensing", for: UIControlState.normal)
            }
            else{
                button.setTitle("Disable BackEMF Sensing", for: UIControlState.normal)
            }
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
        
    }
    
    
    //MARK: navigation
    override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
        if segue.identifier == "infoBoxSegue" {
            phidgetInfoBox = segue.destination as! PhidgetInfoBoxViewController
        }
    }
    
    //MARK: Misc
    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
    }
    
    func updateGUI(backEMFIsHidden:Bool, brakingStrengthIsHidden:Bool, currentLimitIsHidden:Bool, currentRegulatorIsHidden:Bool, fanStateIsHidden:Bool){
        DispatchQueue.main.async {
            self.backEMFTitle.isHidden = backEMFIsHidden
            self.backEMFLabel.isHidden = backEMFIsHidden
            self.enableBackEMFButton.isHidden = backEMFIsHidden
            
            self.brakingStrengthTitle.isHidden = brakingStrengthIsHidden
            self.brakingStrengthLabel.isHidden = brakingStrengthIsHidden
            self.brakingStrengthSlider.isHidden = brakingStrengthIsHidden
            
            self.currentLimitTitle.isHidden = currentLimitIsHidden
            self.currentLimitLabel.isHidden = currentLimitIsHidden
            self.currentLimitSlider.isHidden = currentLimitIsHidden
            
            self.currentRegulatorTitle.isHidden = currentRegulatorIsHidden
            self.currentRegulatorLabel.isHidden = currentRegulatorIsHidden
            self.currentRegulatorSlider.isHidden = currentRegulatorIsHidden
            
            self.fanModeSegmentedControl.isHidden = fanStateIsHidden
        }
    }
}

